/**************************************************************************
* iptrk - ip tracker
* Copyright (C) 2003 Jason Nunn <jsno@leal.com.au>
* Darwin, NT, Australia.
* Originally developed for Arafura Connect (Dec 2000 - Mar 2003)
* Donated to GPL before liquidation.
*
* =======================================================================
* maintenance code (watch)
*
**************************************************************************/
#include <curses.h>
#include "defs.h"
#include "lib.p"
#include "iptrk.p"

PRIVATE watch_t *watch;
PRIVATE S_64    watch_total;

/**************************************************************************
*
**************************************************************************/
PRIVATE void scan_port_prot_tree(treap_t *t)
{
  port_t *n;

  if(t == NULL)
    return;

  scan_port_prot_tree(t->l);

  n = TREAP_ENTRY(t);
  if(watch->service == 0)
    watch_total += n->vol;
  else
    switch(watch->dir)  /*scan port selection*/
    {
      case DIR_I:
      case DIR_i:
        if(watch->service == n->port.port.d)
          watch_total += n->vol;
        break;

      case DIR_O:
      case DIR_o:
        if(watch->service == n->port.port.s)
          watch_total += n->vol;
        break;

      case DIR_u:
        if(watch->service == n->port.port.d ||
           watch->service == n->port.port.s)
        {
          watch_total += n->vol;
        }
        break;
    }

  scan_port_prot_tree(t->r);
}

/**************************************************************************
*
**************************************************************************/
PRIVATE void scan_ports(iptrk_t *n)
{
  switch(watch->prot)
  {
    case PROT_TCP:
      scan_port_prot_tree(n->tcp_port_ll);
      break;

    case PROT_UDP:
      scan_port_prot_tree(n->udp_port_ll);
      break;

    case PROT_OTH:
      scan_port_prot_tree(n->oth_port_ll);
      break;

    default:
      scan_port_prot_tree(n->tcp_port_ll);
      scan_port_prot_tree(n->udp_port_ll);
      scan_port_prot_tree(n->oth_port_ll);
      break;
  }
}

/**************************************************************************
*
**************************************************************************/
PRIVATE void scan_tree(treap_t *t)
{
  iptrk_t *n;
  int     p_dir;

  if(t == NULL)
    return;

  scan_tree(t->l);

  n = TREAP_ENTRY(t);
  if(watch->addr.addr == (n->addr.addr.d & watch->addr.mask) ||
     watch->addr.addr == (n->addr.addr.s & watch->addr.mask))
  {
    p_dir = cast_volume_monitor(n->addr.addr.s, n->addr.addr.d, &(watch->addr));
    switch(watch->dir)
    {
      case DIR_I:
        if(p_dir == DIR_I)
          scan_ports(n);
        break;

      case DIR_O:
        if(p_dir == DIR_O)
          scan_ports(n);
        break;

      case DIR_u:
        if(p_dir == DIR_u)
          scan_ports(n);
        break;

      case DIR_i:
        if(p_dir == DIR_i)
          scan_ports(n);
        break;

      case DIR_o:
        if(p_dir == DIR_o)
          scan_ports(n);
        break;
    }
  }

  scan_tree(t->r);
}

/**************************************************************************
*
**************************************************************************/
PRIVATE void email_watch(int def_f)
{
  char           str[STR_LEN + 1],
                 email_body[STR_LEN_HUGE]; /*hope this is enough ;)*/
  int            tg = 0;
  struct in_addr in;

  if(watch->cond == COND_MORE)
  {
    if(watch_total >= watch->limit)
      tg = 1;
  }
  else
  {
    if(watch_total <= watch->limit)
      tg = 1;
  }

  if(tg)
  {
    in.s_addr = htonl(watch->addr.addr);
    if(def_f)
      sprintf(str, " default watch %s (#%d) trig- %llu (%c %llu)",
        inet_ntoa(in),
        watch->no,
        watch_total,
        watch->cond == COND_MORE ? '>' : '<',
        watch->limit);
    else
      sprintf(str, " watch %s (#%d) trig- %llu (%c %llu)",
        inet_ntoa(in),
        watch->no,
        watch_total,
        watch->cond == COND_MORE ? '>' : '<',
        watch->limit);
    log_m('I', NULL, str);
    compose_watch_email_body(email_body, watch->str, inet_ntoa(in));
    email_operator(str, email_body);
  }
}

/**************************************************************************
***************************************************************************
* WATCH
***************************************************************************
**************************************************************************/
PUBLIC void do_watch(void)
{
  ll_t *ll;

  log_m('I', NULL, " doing watches..");
  ll = boot.watch_ll;
  for(;;)
  {
    watch = ll_read_entry(&ll);
    if(watch == NULL)
      break;

    if(!test_zone(watch->zone))
      continue;

    watch_total = 0;
    scan_tree(watch_bt);
    email_watch(0);
  }
}

/**************************************************************************
***************************************************************************
* LOCAL WATCHES
***************************************************************************
**************************************************************************/

/**************************************************************************
* if there exists an identical watch and local watch entry (accept a 
* different volume limit), then don't do local watch test for that address
**************************************************************************/
PRIVATE int watch_override(watch_t *w)
{
  ll_t           *ll;
  watch_t        *e;

  ll = boot.watch_ll;
  for(;;)
  {
    e = ll_read_entry(&ll);
    if(e == NULL)
      break;

    if(e->addr.addr == (w->addr.addr & e->addr.mask) &&
       e->dir == w->dir &&
       e->cond == w->cond &&
       e->prot == w->prot &&
       e->service == w->service &&
       e->zone == w->zone)
    {
      return 1;
    }
  } 

  return 0;
}

/**************************************************************************
* test each local address
**************************************************************************/
PRIVATE void scan_local_ips(treap_t *t)
{
  if(t == NULL)
    return;

  scan_local_ips(t->l);

  watch->addr.addr = *(U_32 *)TREAP_ENTRY(t);
  if(!watch_override(watch))
  {
    watch_total = 0;
    scan_tree(watch_bt);
    email_watch(1);
  }

  scan_local_ips(t->r);
}

/**************************************************************************
*
**************************************************************************/
PUBLIC void do_watch_default(void)
{
  ll_t *ll;

  log_m('I', NULL, " doing default watches..");
  ll = boot.watch_default_ll;
  for(;;)
  {
    watch = ll_read_entry(&ll);
    if(watch == NULL)
      break;

    if(!test_zone(watch->zone))
      continue;

    watch->addr.mask = 0xffffffff;
    scan_local_ips(our_net_ip_bt);
  }
}
